"""
A Python interface for Discount, the C Markdown parser

This module contains `libmarkdown`, a ctypes binding for Discount,
as well as `Markdown`, a helper class built on top of this library.

Visit the Discount homepage:
http://www.pell.portland.or.us/~orc/Code/discount/

Basic usage examples:

    >>> md = Markdown('`test`')
    >>> md.get_html_content()
    '<p><code>test</code></p>'

    >>> md = Markdown(sys.stdin, autolink=True)
    >>> md.write_html_content(sys.stdout)

See the `Markdown` docstrings for all keyword arguments, or the
docstrings for `libmarkdown` if you want to use the C functions
directly.
"""

import ctypes

import libmarkdown


_KWARGS_TO_LIBMARKDOWN_FLAGS = {
    'toc': libmarkdown.MKD_TOC,
    'strict': libmarkdown.MKD_STRICT,
    'autolink': libmarkdown.MKD_AUTOLINK,
    'safelink': libmarkdown.MKD_SAFELINK,
    'ignore_header': libmarkdown.MKD_NOHEADER,
    'ignore_links': libmarkdown.MKD_NOLINKS,
    'ignore_images': libmarkdown.MKD_NOIMAGE,
    'ignore_tables': libmarkdown.MKD_NOTABLES,
    'ignore_smartypants': libmarkdown.MKD_NOPANTS,
    'ignore_embedded_html': libmarkdown.MKD_NOHTML,
    'ignore_pseudo_protocols': libmarkdown.MKD_NO_EXT,
}


class MarkdownError(Exception):
    """
    Exception raised when a discount c function
    returns an error code, `-1`.
    """
    def __str__(self):
        return '%s failure' % self.args[0]


class Markdown(object):
    """
    Markdown to HTML conversion.

    A single argument is required, `input_file_or_string`, the
    Markdown formatted data.  If this argument is a file-like object,
    the file must be a real OS file descriptor, i.e. `sys.stdin` yes,
    a `StringIO` object, no.  The argument is otherwise assumed to be
    a string-like object.  The same is true for `Markdown` methods
    that write HTML output to files.

    Additional boolean keyword arguments are also accepted:

    * `toc`:
         Generate table-of-contents headers (each generated <h1>,
         <h2>, etc will include a id="name" argument.)  Use
         `get_html_toc()` or `write_html_toc()` to generate the
         table-of-contents itself.

    * `strict`:
         Disable relaxed emphasis and superscripts.

    * `autolink`:
         Greedily expand links; if a url is encountered, convert it to
         a hyperlink even if it isn't surrounded with `<>s`.

    * `safelink`:
         Be paranoid about how `[][]` is expanded into a link - if the
         url isn't a local reference, `http://`, `https://`, `ftp://`,
         or `news://`, it will not be converted into a hyperlink.

    * `ignore_header`:
         Do not process the document header, but treat it like regular
         text.  See http://johnmacfarlane.net/pandoc/README.html#title-blocks

    * `ignore_links`:
         Do not allow `<a` or expand `[][]` into a link.

    * `ignore_images`:
         Do not allow `<img` or expand `![][]` into a image.

    * `ignore_tables`:
         Don't process PHP Markdown Extra tables.  See
         http://michelf.com/projects/php-markdown/extra/.

    * `ignore_smartypants`:
         Disable SmartyPants processing.  See
         http://daringfireball.net/projects/smartypants/.

    * `ignore_embedded_html`:
         Disable all embedded HTML by replacing all `<`'s with `&lt;`.

    * `ignore_pseudo_protocols`:
         Do not process pseudo-protocols.  See
         http://www.pell.portland.or.us/~orc/Code/discount/#pseudo

    Pandoc header elements can be retrieved with the methods
    `get_pandoc_title()`, `get_pandoc_author()` and
    `get_pandoc_date()`.

    The converted HTML document parts can be retrieved as a string
    with the `get_html_css()`, `get_html_toc()` and
    `get_html_content()` methods, or written to a file with the
    `write_html_css(fp)`, `write_html_toc(fp)` and
    `write_html_content(fp)` methods, where `fp` is the output file
    descriptor.
    """
    def __init__(self, input_file_or_string, **kwargs):
        self.input = input_file_or_string

        # Convert a `kwargs` dict to a bitmask of libmarkdown flags.
        # All but one flag is exposed; MKD_1_COMPAT, which, according
        # to the original documentation, is not really useful other
        # than running MarkdownTest_1.0
        flags = 0
        for key in kwargs:
            flags |= _KWARGS_TO_LIBMARKDOWN_FLAGS.get(key, 0)
        self.flags = flags

    def __del__(self):
        try:
            libmarkdown.mkd_cleanup(self._doc)
        except AttributeError:
            pass

    def _get_compiled_doc(self):
        try:
            self._doc
        except AttributeError:
            if hasattr(self.input, 'read'):
                # If the input is file-like
                input_ = ctypes.pythonapi.PyFile_AsFile(self.input)
                self._doc = libmarkdown.mkd_in(input_, self.flags)
            else:
                # Otherwise, treat it as a string
                input_ = ctypes.c_char_p(self.input)
                self._doc = libmarkdown.mkd_string(
                    input_, len(self.input), self.flags)

            ret = libmarkdown.mkd_compile(self._doc, self.flags)
            if ret == -1:
                raise MarkdownError('mkd_compile')

        finally:
            return self._doc

    def _generate_html_content(self, fp=None):
        if fp is not None:
            fp_ = ctypes.pythonapi.PyFile_AsFile(fp)
            ret = libmarkdown.mkd_generatehtml(self._get_compiled_doc(), fp_)
            if ret == -1:
                raise MarkdownError('mkd_generatehtml')
        else:
            sb = ctypes.c_char_p('')
            ln = libmarkdown.mkd_document(self._get_compiled_doc(), ctypes.byref(sb))
            if ln == -1:
                raise MarkdownError('mkd_document')
            else:
                return sb.value[:ln]

    def _generate_html_toc(self, fp=None):
        self.flags |= libmarkdown.MKD_TOC

        if fp is not None:
            fp_ = ctypes.pythonapi.PyFile_AsFile(fp)
            ret = libmarkdown.mkd_generatetoc(self._get_compiled_doc(), fp_)
            if ret == -1:
                raise MarkdownError('mkd_generatetoc')
        else:
            sb = ctypes.c_char_p('')
            ln = libmarkdown.mkd_toc(self._get_compiled_doc(), ctypes.byref(sb))
            if ln == -1:
                raise MarkdownError('mkd_toc')
            else:
                return sb.value[:ln]

    def _generate_html_css(self, fp=None):
        if fp is not None:
            fp_ = ctypes.pythonapi.PyFile_AsFile(fp)
            ret = libmarkdown.mkd_generatecss(self._get_compiled_doc(), fp_)

            # Returns -1 even on success
            # if ret == -1:
            #     raise MarkdownError('mkd_generatecss')
        else:
            sb = ctypes.c_char_p('')
            ln = libmarkdown.mkd_css(self._get_compiled_doc(), ctypes.byref(sb))

            if ln == -1:
                raise MarkdownError('mkd_css')
            else:
                return sb.value[:ln]

    def get_pandoc_title(self):
        """
        Get the document title from the pandoc header.
        """
        return libmarkdown.mkd_doc_title(self._get_compiled_doc())

    def get_pandoc_author(self):
        """
        Get the document author(s) from the pandoc header.
        """
        return libmarkdown.mkd_doc_author(self._get_compiled_doc())

    def get_pandoc_date(self):
        """
        Get the document date from the pandoc header.
        """
        return libmarkdown.mkd_doc_date(self._get_compiled_doc())

    def get_html_content(self):
        """
        Get the document content as HTML.
        """
        return self._generate_html_content()

    def get_html_toc(self):
        """
        Get the document's table of contents as HTML.
        """
        return self._generate_html_toc()

    def get_html_css(self):
        """
        Get any style blocks in the document as HTML.
        """
        return self._generate_html_css()

    def write_html_content(self, fp):
        """
        Write the document content to the file, `fp`.
        """
        self._generate_html_content(fp)

    def write_html_toc(self, fp):
        """
        Write the document's table of contents to the file, `fp`.
        """
        self._generate_html_toc(fp)

    def write_html_css(self, fp):
        """
        Write any style blocks in the document to the file, `fp`.
        """
        self._generate_html_css(fp)



import ctypes
import tempfile
import unittest

from discount import Markdown, libmarkdown


class LibmarkdownTestCase(unittest.TestCase):
    def test_pythonapi(self):
        self.assertEqual(
            ctypes.pythonapi.PyFile_AsFile.argtypes,
            (ctypes.py_object,)
        )

        self.assertEqual(
            ctypes.pythonapi.PyFile_AsFile.restype,
            ctypes.POINTER(libmarkdown.FILE),
        )

    def test_mkd_in(self):
        self.assertEqual(
            libmarkdown.mkd_in.argtypes,
            (ctypes.POINTER(libmarkdown.FILE), ctypes.c_int),
        )

        self.assertEqual(
            libmarkdown.mkd_in.restype,
            ctypes.POINTER(libmarkdown.Document),
        )

        i = tempfile.TemporaryFile('r+w')
        i.write('`test`')
        i.seek(0)
        i_ = ctypes.pythonapi.PyFile_AsFile(i)
        libmarkdown.mkd_in(i_, 0)
        i.close()

    def test_mkd_string(self):
        self.assertEqual(
            libmarkdown.mkd_string.argtypes,
            (ctypes.c_char_p, ctypes.c_int, ctypes.c_int)
        )

        self.assertEqual(
            libmarkdown.mkd_string.restype,
            ctypes.POINTER(libmarkdown.Document)
        )

        text = '`test`'
        cp = ctypes.c_char_p(text)
        libmarkdown.mkd_string(cp, len(text), 0)

    def test_markdown(self):
        self.assertEqual(
            libmarkdown.markdown.argtypes,
            (ctypes.POINTER(libmarkdown.Document),
             ctypes.POINTER(libmarkdown.FILE),
             ctypes.c_int)
        )

        self.assertEqual(
            libmarkdown.markdown.restype,
            ctypes.c_int,
        )

        text = '`test`'
        cp = ctypes.c_char_p(text)
        doc = libmarkdown.mkd_string(cp, len(text), 0)
        out = tempfile.TemporaryFile('r+w')
        ret = libmarkdown.markdown(
            doc, ctypes.pythonapi.PyFile_AsFile(out), 0
        )
        out.seek(0)
        html = out.read()

        self.assertNotEqual(ret, -1)
        self.assertEqual(html, '<p><code>test</code></p>\n')

        out.close()

    def test_mkd_line(self):
        self.assertEqual(
            libmarkdown.mkd_line.argtypes,
            (ctypes.c_char_p,
             ctypes.c_int,
             ctypes.POINTER(ctypes.c_char_p),
             ctypes.c_int),
        )

        self.assertEqual(
            libmarkdown.mkd_line.restype,
            ctypes.c_int,
        )

        text = '`test`'
        cp = ctypes.c_char_p(text)
        sb = ctypes.c_char_p('')
        ret = libmarkdown.mkd_line(cp, len(text), ctypes.byref(sb), 0)

        self.assertNotEqual(ret, -1)
        self.assertEqual(sb.value, '<code>test</code>')

    def test_mkd_generateline(self):
        self.assertEqual(
            libmarkdown.mkd_generateline.argtypes,
            (ctypes.c_char_p,
             ctypes.c_int,
             ctypes.POINTER(libmarkdown.FILE),
             ctypes.c_int),
        )

        self.assertEqual(
            libmarkdown.mkd_generateline.restype,
            ctypes.c_int,
        )

        text = '`test`'
        cp = ctypes.c_char_p(text)
        out = tempfile.TemporaryFile('r+w')
        ret = libmarkdown.mkd_generateline(
            cp, len(text), ctypes.pythonapi.PyFile_AsFile(out), 0
        )

        self.assertNotEqual(ret, -1)

        out.close()

    def test_mkd_compile(self):
        self.assertEqual(
            libmarkdown.mkd_compile.argtypes,
            (ctypes.POINTER(libmarkdown.Document), ctypes.c_int),
        )

        self.assertEqual(
            libmarkdown.mkd_compile.restype,
            ctypes.c_int,
        )

        text = '`test`'
        cp = ctypes.c_char_p(text)
        doc = libmarkdown.mkd_string(cp, len(text), 0)
        ret = libmarkdown.mkd_compile(doc, 0)

        self.assertNotEqual(ret, -1)
        self.assertEqual(doc.contents.compiled, 1)

        libmarkdown.mkd_cleanup(doc)

    def test_mkd_generatehtml(self):
        self.assertEqual(
            libmarkdown.mkd_generatehtml.argtypes,
            (ctypes.POINTER(libmarkdown.Document),
             ctypes.POINTER(libmarkdown.FILE)),
        )

        self.assertEqual(
            libmarkdown.mkd_generatehtml.restype,
            ctypes.c_int,
        )

        text = '`test`'
        cp = ctypes.c_char_p(text)
        out = tempfile.TemporaryFile('r+w')
        doc = libmarkdown.mkd_string(cp, len(text), 0)
        ret = libmarkdown.mkd_compile(doc, 0)

        self.assertNotEqual(ret, -1)

        ret = libmarkdown.mkd_generatehtml(
            doc, ctypes.pythonapi.PyFile_AsFile(out), 0
        )

        self.assertNotEqual(ret, -1)

        out.seek(0)
        html = out.read()

        self.assertEqual(
            html, '<p><code>test</code></p>\n',
        )

        libmarkdown.mkd_cleanup(doc)

        out.close()

    def test_mkd_document(self):
        self.assertEqual(
            libmarkdown.mkd_document.argtypes,
            (ctypes.POINTER(libmarkdown.Document),
             ctypes.POINTER(ctypes.c_char_p)),
        )

        self.assertEqual(
            libmarkdown.mkd_document.restype,
            ctypes.c_int,
        )

        text = '`test`'
        cp = ctypes.c_char_p(text)
        sb = ctypes.c_char_p('')
        doc = libmarkdown.mkd_string(cp, len(text), 0)
        ret = libmarkdown.mkd_compile(doc, 0)

        self.assertNotEqual(ret, -1)

        ret = libmarkdown.mkd_document(doc, ctypes.byref(sb), 0)

        self.assertNotEqual(ret, -1)
        self.assertEqual(
            sb.value[:ret],
            '<p><code>test</code></p>'
        )

    def test_mkd_css(self):
        self.assertEqual(
            libmarkdown.mkd_css.argtypes,
            (ctypes.POINTER(libmarkdown.Document),
             ctypes.POINTER(ctypes.c_char_p)),
        )

        self.assertEqual(
            libmarkdown.mkd_css.restype,
            ctypes.c_int,
        )

        text = '<style>\n  *{color:red}\n</style>\n\n# header-1\n\n`test`'
        cp = ctypes.c_char_p(text)
        sb = ctypes.c_char_p('')
        doc = libmarkdown.mkd_string(cp, len(text), 0)
        ret = libmarkdown.mkd_compile(doc, 0)

        self.assertNotEqual(ret, -1)

        ret = libmarkdown.mkd_css(doc, ctypes.byref(sb))

        self.assertNotEqual(ret, -1)
        self.assertEqual(
            sb.value[:ret],
            '<style>  *{color:red}</style>\n'
        )

    def test_mkd_generatecss(self):
        self.assertEqual(
            libmarkdown.mkd_generatecss.argtypes,
            (ctypes.POINTER(libmarkdown.Document),
             ctypes.POINTER(libmarkdown.FILE)),
        )

        self.assertEqual(
            libmarkdown.mkd_generatecss.restype,
            ctypes.c_int,
        )

        text = '<style>\n  *{color:red}\n</style>\n\n# header-1\n\n`test`'
        cp = ctypes.c_char_p(text)
        out = tempfile.TemporaryFile('r+w')
        doc = libmarkdown.mkd_string(cp, len(text), 0)
        ret = libmarkdown.mkd_compile(doc, 0)

        self.assertNotEqual(ret, -1)

        ret = libmarkdown.mkd_generatecss(
            doc, ctypes.pythonapi.PyFile_AsFile(out)
        )

        # Returns -1 event on success
        # self.assertNotEqual(ret, -1)

        out.seek(0)
        html = out.read()

        self.assertEqual(
            html, '<style>  *{color:red}</style>\n'
        )

        out.close()

    def test_mkd_toc(self):
        self.assertEqual(
            libmarkdown.mkd_toc.argtypes,
            (ctypes.POINTER(libmarkdown.Document),
             ctypes.POINTER(ctypes.c_char_p)),
        )

        self.assertEqual(
            libmarkdown.mkd_toc.restype,
            ctypes.c_int,
        )

        flags = libmarkdown.MKD_TOC
        text = '# header-1\n## header-2\n### header-3'
        cp = ctypes.c_char_p(text)
        sb = ctypes.c_char_p('')
        doc = libmarkdown.mkd_string(cp, len(text), flags)
        ret = libmarkdown.mkd_compile(doc, flags)

        self.assertNotEqual(ret, -1)

        ret = libmarkdown.mkd_toc(doc, ctypes.byref(sb))

        self.assertNotEqual(ret, -1)
        self.assertEqual(
            sb.value[:ret], (
                '\n '
                '<ul>\n <li><a href="#header-1">header-1</a>\n  '
                '<ul>\n  <li><a href="#header-2">header-2</a>\n   '
                '<ul>\n   <li><a href="#header-3">header-3</a>   </li>\n   '
                '</ul>\n  </li>\n  </ul>\n </li>\n </ul>\n'
            )
        )

    def test_mkd_generatetoc(self):
        self.assertEqual(
            libmarkdown.mkd_generatetoc.argtypes,
            (ctypes.POINTER(libmarkdown.Document),
             ctypes.POINTER(libmarkdown.FILE)),
        )

        self.assertEqual(
            libmarkdown.mkd_generatetoc.restype,
            ctypes.c_int,
        )

        flags = libmarkdown.MKD_TOC
        text = '# header-1\n## header-2\n### header-3'
        cp = ctypes.c_char_p(text)
        out = tempfile.TemporaryFile('r+w')
        doc = libmarkdown.mkd_string(cp, len(text), flags)
        ret = libmarkdown.mkd_compile(doc, flags)

        self.assertNotEqual(ret, -1)

        ret = libmarkdown.mkd_generatetoc(
            doc, ctypes.pythonapi.PyFile_AsFile(out)
        )

        self.assertNotEqual(ret, -1)

        out.seek(0)
        html = out.read()

        self.assertEqual(
            html, (
                '\n '
                '<ul>\n <li><a href="#header-1">header-1</a>\n  '
                '<ul>\n  <li><a href="#header-2">header-2</a>\n   '
                '<ul>\n   <li><a href="#header-3">header-3</a>   </li>\n   '
                '</ul>\n  </li>\n  </ul>\n </li>\n </ul>\n'
            )
        )

        out.close()

    def test_mkd_dump(self):
        self.assertEqual(
            libmarkdown.mkd_dump.argtypes,
            (ctypes.POINTER(libmarkdown.Document),
             ctypes.POINTER(libmarkdown.FILE),
             ctypes.c_int,
             ctypes.c_char_p),
        )

        self.assertEqual(
            libmarkdown.mkd_dump.restype,
            ctypes.c_int,
        )

        text = (
            '# header-1\n'
            '`test`\n'
        )

        cp = ctypes.c_char_p(text)

        doc = libmarkdown.mkd_string(cp, len(text), 0)
        ret = libmarkdown.mkd_compile(doc, 0)

        self.assertNotEqual(ret, -1)

        out = tempfile.TemporaryFile('r+w')

        libmarkdown.mkd_dump(
            doc, ctypes.pythonapi.PyFile_AsFile(out),
            0, ctypes.c_char_p('title')
        )

        out.seek(0)
        dump = out.read()

        self.assertEqual(
            dump, (
                'title-----[source]--+--[header, <P>, 1 line]\n'
                '                    `--[markup, <P>, 1 line]\n'
            )
        )

        out.close()

    def test_mkd_cleanup(self):
        self.assertEqual(
            libmarkdown.mkd_cleanup.argtypes,
            (ctypes.POINTER(libmarkdown.Document),)
        )

        self.assertEqual(
            libmarkdown.mkd_cleanup.restype,
            ctypes.c_int,
        )

        text = '`test`'
        cp = ctypes.c_char_p(text)
        doc = libmarkdown.mkd_string(cp, len(text), 0)
        libmarkdown.mkd_compile(doc, 0)

        self.assertEqual(doc.contents.compiled, 1)

        libmarkdown.mkd_cleanup(doc)

        self.assertNotEqual(doc.contents.compiled, 1)

    def test_mkd_doc_title(self):
        self.assertEqual(
            libmarkdown.mkd_doc_title.argtypes,
            (ctypes.POINTER(libmarkdown.Document),)
        )

        self.assertEqual(
            libmarkdown.mkd_doc_title.restype,
            ctypes.c_char_p,
        )

        text = '% title\n% author\n% date\n'
        cp = ctypes.c_char_p(text)
        doc = libmarkdown.mkd_string(cp, len(text), 0)
        ret = libmarkdown.mkd_compile(doc, 0)

        self.assertNotEqual(ret, -1)

        v = libmarkdown.mkd_doc_title(doc)

        self.assertEqual(v, 'title')

    def test_mkd_doc_author(self):
        self.assertEqual(
            libmarkdown.mkd_doc_author.argtypes,
            (ctypes.POINTER(libmarkdown.Document),)
        )

        self.assertEqual(
            libmarkdown.mkd_doc_author.restype,
            ctypes.c_char_p,
        )

        text = '% title\n% author\n% date\n'
        cp = ctypes.c_char_p(text)
        doc = libmarkdown.mkd_string(cp, len(text), 0)
        ret = libmarkdown.mkd_compile(doc, 0)

        self.assertNotEqual(ret, -1)

        v = libmarkdown.mkd_doc_author(doc)

        self.assertEqual(v, 'author')

    def test_mkd_doc_date(self):
        self.assertEqual(
            libmarkdown.mkd_doc_date.argtypes,
            (ctypes.POINTER(libmarkdown.Document),)
        )

        self.assertEqual(
            libmarkdown.mkd_doc_date.restype,
            ctypes.c_char_p,
        )

        text = '% title\n% author\n% date\n'
        cp = ctypes.c_char_p(text)
        doc = libmarkdown.mkd_string(cp, len(text), 0)
        ret = libmarkdown.mkd_compile(doc, 0)

        self.assertNotEqual(ret, -1)

        v = libmarkdown.mkd_doc_date(doc)

        self.assertEqual(v, 'date')

    # The following flag tests just ensure that the flags bits are
    # correct and enable/disable the appropriate output features.

    def _test_flag(self, i, o, f):
        text = i
        cp = ctypes.c_char_p(text)

        doc = libmarkdown.mkd_string(
            cp, len(text), f
        )
        libmarkdown.mkd_compile(doc, f)

        sb = ctypes.c_char_p('')
        ret = libmarkdown.mkd_document(doc, ctypes.byref(sb))

        self.assertEqual(
            sb.value[:ret],
            o,
        )

    def test_MKD_NOLINKS(self):
        self._test_flag(
            '<a href="a">a</a>',
            '<p><a href="a">a</a></p>',
            0
        )

        self._test_flag(
            '<a href="a">a</a>',
            '<p>&lt;a href="a">a</a></p>',
            libmarkdown.MKD_NOLINKS
        )

    def test_MKD_NOIMAGE(self):
        self._test_flag(
            '<img src="test.png" />',
            '<p><img src="test.png" /></p>',
            0
        )

        self._test_flag(
            '<img src="test.png" />',
            '<p>&lt;img src="test.png" /></p>',
            libmarkdown.MKD_NOIMAGE
        )

    def test_MKD_NOHTML(self):
        self._test_flag(
            '<div>test</div>',
            '<div>test</div>\n',
            0
        )

        self._test_flag(
            '<div>test</div>',
            '<p>&lt;div>test&lt;/div></p>',
            libmarkdown.MKD_NOHTML
        )

    def test_MKD_NOPANTS(self):
        self._test_flag(
            (
                '"test"\n'
                '``test''\n'
                'a -- b\n'
                'a --- b\n'
                'a ... b\n'
            ),
            '<p>&ldquo;test&rdquo;\n``test\na &mdash; b\n'
            'a &mdash;&ndash; b\na &hellip; b</p>',
            0
        )

        self._test_flag(
            (
                '"test"\n'
                '``test''\n'
                'a -- b\n'
                'a --- b\n'
                'a ... b\n'
            ),
            '<p>"test"\n``test\na -- b\na --- b\na ... b</p>',
            libmarkdown.MKD_NOPANTS
        )

    def test_MKD_NOHEADER(self):
        self._test_flag(
            (
                '%s title\n'
                '%s author(s)\n'
                '%s date\n'
                '`test`\n'
            ),
            '<p><code>test</code></p>',
            0
        )

        self._test_flag(
            (
                '%s title\n'
                '%s author(s)\n'
                '%s date\n'
                '`test`\n'
            ),
            '<p>%s title\n%s author(s)\n%s date\n<code>test</code></p>',
            libmarkdown.MKD_NOHEADER
        )

    def test_MKD_AUTOLINK(self):
        self._test_flag(
            'http://example.com',
            '<p>http://example.com</p>',
            0
        )

        self._test_flag(
            'http://example.com',
            '<p><a href="http://example.com">http://example.com</a></p>',
            libmarkdown.MKD_AUTOLINK
        )

    def test_MKD_SAFELINK(self):
        self._test_flag(
            (
                '[a][x]\n\n'
                '[x]: file://test "Some title"'
            ),
            '<p><a href="file://test" title="Some title">a</a></p>',
            0
        )

        self._test_flag(
            (
                '[a][x]\n\n'
                '[x]: file://test "Some title"'
            ),
            '<p>[a][x]</p>',
            libmarkdown.MKD_SAFELINK
        )

    def test_MKD_NOTABLES(self):
        self._test_flag(
            'a  | b\n'
            '-- | -\n'
            '1  | 2\n',
            '<table>\n<thead>\n<tr>\n<th>a  </th>\n'
            '<th> b</th>\n</tr>\n</thead>\n<tbody>\n<tr>\n'
            '<td>1  </td>\n<td> 2</td>\n</tr>\n</tbody>\n</table>\n',
            0
        )

        self._test_flag(
            'a  | b\n'
            '-- | -\n'
            '1  | 2\n',
            '<p>a  | b\n&mdash; | &ndash;\n1  | 2</p>',
            libmarkdown.MKD_NOTABLES
        )

    def test_MKD_NO_EXT(self):
        self._test_flag(
            '[foo](class:bar)\n',
            '<p><span class="bar">foo</span></p>',
            0
        )

        self._test_flag(
            '[foo](class:bar)\n',
            '<p>[foo](class:bar)</p>',
            libmarkdown.MKD_NO_EXT
        )

    def test_MKD_STRICT(self):
        self._test_flag(
            'A^B\n',
            '<p>A<sup>B</sup></p>',
            0
        )

        self._test_flag(
            'A^B\n',
            '<p>A^B</p>',
            libmarkdown.MKD_STRICT
        )

    def test_MKD_TOC(self):
        self._test_flag(
            '# header-1\n'
            '## header-2\n',
            '<h1>header-1</h1>\n\n<h2>header-2</h2>',
            0
        )

        self._test_flag(
            '# header-1\n'
            '## header-2\n',
            '<h1 id="header-1">header-1</h1>\n\n'
            '<h2 id="header-2">header-2</h2>',
            libmarkdown.MKD_TOC
        )


class MarkdownClassTestCase(unittest.TestCase):
    def test_fails_without_args(self):
        self.assertRaises(TypeError, Markdown)

    def test__accepts_string_arg(self):
        md = Markdown('`test`')

    def test_accepts_file_arg(self):
        i = tempfile.TemporaryFile('r+w')
        md = Markdown(i)
        i.close()

    def test_kwargs_generates_valid_bitmask(self):
        md = Markdown('`test`')
        self.assertEqual(md.flags, 0)

        md = Markdown('`test`', strict=True)
        self.assertEqual(md.flags, libmarkdown.MKD_STRICT)

        md = Markdown('`test`', autolink=True)
        self.assertEqual(md.flags, libmarkdown.MKD_AUTOLINK)

        md = Markdown('`test`', safelink=True)
        self.assertEqual(md.flags, libmarkdown.MKD_SAFELINK)

        md = Markdown('`test`', ignore_header=True)
        self.assertEqual(md.flags, libmarkdown.MKD_NOHEADER)

        md = Markdown('`test`', ignore_links=True)
        self.assertEqual(md.flags, libmarkdown.MKD_NOLINKS)

        md = Markdown('`test`', ignore_images=True)
        self.assertEqual(md.flags, libmarkdown.MKD_NOIMAGE)

        md = Markdown('`test`', ignore_tables=True)
        self.assertEqual(md.flags, libmarkdown.MKD_NOTABLES)

        md = Markdown('`test`', ignore_smartypants=True)
        self.assertEqual(md.flags, libmarkdown.MKD_NOPANTS)

        md = Markdown('`test`', ignore_embedded_html=True)
        self.assertEqual(md.flags, libmarkdown.MKD_NOHTML)

        md = Markdown('`test`', ignore_pseudo_protocols=True)
        self.assertEqual(md.flags, libmarkdown.MKD_NO_EXT)

    def test_input_string_get_pandoc_title(self):
        md = Markdown('`test`')
        self.assertEqual(md.get_pandoc_title(), None)

        md = Markdown('% abc\n')
        self.assertEqual(md.get_pandoc_title(), None)

        md = Markdown('% abc\n% def')
        self.assertEqual(md.get_pandoc_title(), None)

        md = Markdown('% abc\n% def\n% jhi\n')
        self.assertEqual(md.get_pandoc_title(), 'abc')

    def test_input_string_get_pandoc_author(self):
        md = Markdown('`test`')
        self.assertEqual(md.get_pandoc_author(), None)

        md = Markdown('% abc\n')
        self.assertEqual(md.get_pandoc_author(), None)

        md = Markdown('% abc\n% def')
        self.assertEqual(md.get_pandoc_author(), None)

        md = Markdown('% abc\n% def\n% jhi\n')
        self.assertEqual(md.get_pandoc_author(), 'def')

    def test_input_string_get_pandoc_date(self):
        md = Markdown('`test`')
        self.assertEqual(md.get_pandoc_date(), None)

        md = Markdown('% abc\n')
        self.assertEqual(md.get_pandoc_date(), None)

        md = Markdown('% abc\n% def')
        self.assertEqual(md.get_pandoc_date(), None)

        md = Markdown('% abc\n% def\n% jhi\n')
        self.assertEqual(md.get_pandoc_date(), 'jhi')

    def test_input_string_get_html_css(self):
        md = Markdown(
            '<style>\n  *{color:red}\n</style>\n\n# header-1\n\n`test`')

        style = md.get_html_css()

        self.assertEqual(style, '<style>  *{color:red}</style>\n')

    def test_input_string_get_html_toc(self):
        md = Markdown('# header-1\n## header-2\n### header-3')
        html = md.get_html_toc()

        self.assertEqual(
            html, (
                '\n '
                '<ul>\n <li><a href="#header-1">header-1</a>\n  '
                '<ul>\n  <li><a href="#header-2">header-2</a>\n   '
                '<ul>\n   <li><a href="#header-3">header-3</a>   </li>\n   '
                '</ul>\n  </li>\n  </ul>\n </li>\n </ul>\n'
            )
        )

    def test_input_string_get_html_content(self):
        md = Markdown('`test`')

        html = md.get_html_content()

        self.assertEqual(html, '<p><code>test</code></p>')

    def test_input_string_write_html_css(self):
        o = tempfile.TemporaryFile('r+w')
        md = Markdown(
            '<style>\n  *{color:red}\n</style>\n\n# header-1\n\n`test`')
        md.write_html_css(o)

        o.seek(0)
        style = o.read()
        o.close()

        self.assertEqual(style, '<style>  *{color:red}</style>\n')

    def test_input_string_write_html_toc(self):
        o = tempfile.TemporaryFile('r+w')
        md = Markdown('# header-1\n## header-2\n### header-3')
        md.write_html_toc(o)

        o.seek(0)
        html = o.read()
        o.close()

        self.assertEqual(
            html, (
                '\n '
                '<ul>\n <li><a href="#header-1">header-1</a>\n  '
                '<ul>\n  <li><a href="#header-2">header-2</a>\n   '
                '<ul>\n   <li><a href="#header-3">header-3</a>   </li>\n   '
                '</ul>\n  </li>\n  </ul>\n </li>\n </ul>\n'
            )
        )

    def test_input_string_write_html_content(self):
        o = tempfile.TemporaryFile('r+w')
        md = Markdown('`test`')
        md.write_html_content(o)

        o.seek(0)
        html = o.read()
        o.close()

        self.assertEqual(html, '<p><code>test</code></p>\n')

    def test_input_file_get_pandoc_title(self):
        i = tempfile.TemporaryFile('r+w')
        i.write('`test`')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_title(), None)
        i.close()

        i = tempfile.TemporaryFile('r+w')
        i.write('% abc\n')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_title(), None)
        i.close()

        i = tempfile.TemporaryFile('r+w')
        i.write('% abc\n% def')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_title(), None)
        i.close()

        i = tempfile.TemporaryFile('r+w')
        i.write('% abc\n% def\n% jhi\n')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_title(), 'abc')
        i.close()

    def test_input_file_get_pandoc_author(self):
        i = tempfile.TemporaryFile('r+w')
        i.write('`test`')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_author(), None)
        i.close()

        i = tempfile.TemporaryFile('r+w')
        i.write('% abc\n')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_author(), None)
        i.close()

        i = tempfile.TemporaryFile('r+w')
        i.write('% abc\n% def')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_author(), None)
        i.close()

        i = tempfile.TemporaryFile('r+w')
        i.write('% abc\n% def\n% jhi\n')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_author(), 'def')
        i.close()

    def test_input_file_get_pandoc_date(self):
        i = tempfile.TemporaryFile('r+w')
        i.write('`test`')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_date(), None)
        i.close()

        i = tempfile.TemporaryFile('r+w')
        i.write('% abc\n')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_date(), None)
        i.close()

        i = tempfile.TemporaryFile('r+w')
        i.write('% abc\n% def')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_date(), None)
        i.close()

        i = tempfile.TemporaryFile('r+w')
        i.write('% abc\n% def\n% jhi\n')
        i.seek(0)
        md = Markdown(i)
        self.assertEqual(md.get_pandoc_date(), 'jhi')
        i.close()

    def test_input_file_get_html_css(self):
        i = tempfile.TemporaryFile('r+w')
        i.write('<style>\n  *{color:red}\n</style>\n\n# header-1\n\n`test`')
        i.seek(0)
        md = Markdown(i)

        style = md.get_html_css()

        self.assertEqual(style, '<style>  *{color:red}</style>\n')

    def test_input_file_get_html_toc(self):
        i = tempfile.TemporaryFile('r+w')
        i.write('# header-1\n## header-2\n### header-3')
        i.seek(0)
        md = Markdown(i)
        html = md.get_html_toc()
        i.close()

        self.assertEqual(
            html, (
                '\n '
                '<ul>\n <li><a href="#header-1">header-1</a>\n  '
                '<ul>\n  <li><a href="#header-2">header-2</a>\n   '
                '<ul>\n   <li><a href="#header-3">header-3</a>   </li>\n   '
                '</ul>\n  </li>\n  </ul>\n </li>\n </ul>\n'
            )
        )

    def test_input_file_get_html_content(self):
        i = tempfile.TemporaryFile('r+w')
        i.write('`test`')
        i.seek(0)
        md = Markdown(i)
        html = md.get_html_content()
        i.close()

        self.assertEqual(html, '<p><code>test</code></p>')

    def test_input_file_write_html_css(self):
        i = tempfile.TemporaryFile('r+w')
        i.write('<style>\n  *{color:red}\n</style>\n\n# header-1\n\n`test`')
        i.seek(0)
        o = tempfile.TemporaryFile('r+w')
        md = Markdown(i)
        md.write_html_css(o)

        o.seek(0)
        style = o.read()

        o.close()
        i.close()

        self.assertEqual(style, '<style>  *{color:red}</style>\n')

    def test_input_file_write_html_toc(self):
        i = tempfile.TemporaryFile('r+w')
        i.write('# header-1\n## header-2\n### header-3')
        i.seek(0)
        o = tempfile.TemporaryFile('r+w')
        md = Markdown(i)
        md.write_html_toc(o)

        o.seek(0)
        html = o.read()

        i.close()
        o.close()

        self.assertEqual(
            html, (
                '\n '
                '<ul>\n <li><a href="#header-1">header-1</a>\n  '
                '<ul>\n  <li><a href="#header-2">header-2</a>\n   '
                '<ul>\n   <li><a href="#header-3">header-3</a>   </li>\n   '
                '</ul>\n  </li>\n  </ul>\n </li>\n </ul>\n'
            )
        )

    def test_input_file_write_html_content(self):
        i = tempfile.TemporaryFile('r+w')
        i.write('`test`')
        i.seek(0)
        o = tempfile.TemporaryFile('r+w')
        md = Markdown(i)
        md.write_html_content(o)

        o.seek(0)
        html = o.read()

        i.close()
        o.close()

        self.assertEqual(html, '<p><code>test</code></p>\n')


if __name__ == '__main__':
    unittest.main()



"""
Python bindings for Discount

The `libmarkdown` public functions accept `FILE*` and `char**` types
as their arguments, which require some additional ctypes boilerplate.

To obtain a `FILE*` from a Python file descriptor for use with
`libmarkdown`, use the following pattern:

    i = ctypes.pythonapi.PyFile_AsFile(sys.stdin)
    o = ctypes.pythonapi.PyFile_AsFile(sys.stdout)
    doc = libmarkdown.mkd_in(i)
    libmarkdown.markdown(doc, o)

For `libmarkdown` functions to which you pass a `char**`, use the
following pattern:

    cp = ctypes.c_char_p('')
    ln = libmarkdown.mkd_document(doc, ctypes.byref(cp))
    html_text = cp.value[:ln]

It is important to initialize `c_char_p` with an empty string.
"""

import ctypes
import os


MKD_NOLINKS = 0x0001
MKD_NOIMAGE = 0x0002
MKD_NOPANTS = 0x0004
MKD_NOHTML = 0x0008
MKD_STRICT = 0x0010
MKD_NO_EXT = 0x0040
MKD_NOHEADER = 0x0100
MKD_TABSTOP = 0x0200
MKD_NOTABLES = 0x0400
MKD_TOC = 0x1000
MKD_1_COMPAT = 0x2000
MKD_AUTOLINK =  0x4000
MKD_SAFELINK = 0x8000


_so = ctypes.CDLL(
    os.path.join(
        os.path.dirname(os.path.dirname(os.path.abspath(__file__))),
        '_discount.so'
    )
)


markdown_version = (ctypes.c_char * 64).in_dll(_so, 'markdown_version').value


class FILE(ctypes.Structure):
    pass


ctypes.pythonapi.PyFile_AsFile.argtypes = (ctypes.py_object,)

ctypes.pythonapi.PyFile_AsFile.restype = ctypes.POINTER(FILE)


class Cstring(ctypes.Structure):
    _fields_ = [
        ('text', ctypes.c_char_p),
        ('size', ctypes.c_int),
        ('alloc', ctypes.c_int),
    ]


class Line(ctypes.Structure):
    pass

Line._fields_ = [
    ('text', Cstring),
    ('next', ctypes.POINTER(Line)),
    ('dle', ctypes.c_int)
]


class Line_ANCHOR(ctypes.Structure):
    _fields_ = [
        ('text', ctypes.POINTER(Line)),
        ('end', ctypes.POINTER(Line)),
    ]


class Paragraph(ctypes.Structure):
    pass

Paragraph._fields_ = [
    ('next', ctypes.POINTER(Paragraph)),
    ('down', ctypes.POINTER(Paragraph)),
    ('text', ctypes.POINTER(Line)),
    ('ident', ctypes.c_char_p),
    ('typ', ctypes.c_int),
    ('align', ctypes.c_int),
    ('hnumber', ctypes.c_int),
]


class Block(ctypes.Structure):
    _fields_ = [
        ('b_type', ctypes.c_int),
        ('b_count', ctypes.c_int),
        ('b_char', ctypes.c_char),
        ('b_text', Cstring),
        ('b_post', Cstring),
    ]


class Qblock(ctypes.Structure):
    _fields_ = [
        ('text', ctypes.POINTER(Block)),
        ('size', ctypes.c_int),
        ('alloc', ctypes.c_int),
    ]


class Footnote(ctypes.Structure):
    _fields_ = [
        ('tag', Cstring),
        ('link', Cstring),
        ('title', Cstring),
        ('height', ctypes.c_int),
        ('width', ctypes.c_int),
        ('dealloc', ctypes.c_int),
    ]


class Footnote_STRING(ctypes.Structure):
    _fields_ = [
        ('text', ctypes.POINTER(Footnote)),
        ('size', ctypes.c_int),
        ('alloc', ctypes.c_int),
    ]


class MMIOT(ctypes.Structure):
    _fields_ = [
        ('out', Cstring),
        ('in_', Cstring),
        ('Q', Qblock),
        ('isp', ctypes.c_int),
        ('footnotes', ctypes.POINTER(Footnote_STRING)),
        ('flags', ctypes.c_int),
        ('base', ctypes.c_char_p),
    ]


class Document(ctypes.Structure):
    _fields_ = [
        ('headers', ctypes.POINTER(Line)),
        ('content', Line_ANCHOR),
        ('code', ctypes.POINTER(Paragraph)),
        ('compiled', ctypes.c_int),
        ('html', ctypes.c_int),
        ('tabstop', ctypes.c_int),
        ('ctx', ctypes.POINTER(MMIOT)),
        ('base', ctypes.c_char_p),
    ]


mkd_in = _so.mkd_in
mkd_in.argtypes = (
    ctypes.POINTER(FILE),
    ctypes.c_int,
)
mkd_in.restype = ctypes.POINTER(Document)

mkd_string = _so.mkd_string
mkd_string.argtypes = (
    ctypes.c_char_p,
    ctypes.c_int,
    ctypes.c_int,
)
mkd_string.restype = ctypes.POINTER(Document)

markdown = _so.markdown
markdown.argtypes = (
    ctypes.POINTER(Document),
    ctypes.POINTER(FILE),
    ctypes.c_int,
)

mkd_line = _so.mkd_line
mkd_line.argtypes = (
    ctypes.c_char_p,
    ctypes.c_int,
    ctypes.POINTER(ctypes.c_char_p),
    ctypes.c_int,
)

mkd_generateline = _so.mkd_generateline
mkd_generateline.argtypes = (
    ctypes.c_char_p,
    ctypes.c_int,
    ctypes.POINTER(FILE),
    ctypes.c_int,
)

mkd_compile = _so.mkd_compile
mkd_compile.argtypes = (
    ctypes.POINTER(Document),
    ctypes.c_int,
)

mkd_generatehtml = _so.mkd_generatehtml
mkd_generatehtml.argtypes = (
    ctypes.POINTER(Document),
    ctypes.POINTER(FILE),
)

mkd_document = _so.mkd_document
mkd_document.argtypes = (
    ctypes.POINTER(Document),
    ctypes.POINTER(ctypes.c_char_p),
)

mkd_css = _so.mkd_css
mkd_css.argtypes = (
    ctypes.POINTER(Document),
    ctypes.POINTER(ctypes.c_char_p),
)

mkd_generatecss = _so.mkd_generatecss
mkd_generatecss.argtypes = (
    ctypes.POINTER(Document),
    ctypes.POINTER(FILE),
)

mkd_toc = _so.mkd_toc
mkd_toc.argtypes = (
    ctypes.POINTER(Document),
    ctypes.POINTER(ctypes.c_char_p),
)

mkd_generatetoc = _so.mkd_generatetoc
mkd_generatetoc.argtypes = (
    ctypes.POINTER(Document),
    ctypes.POINTER(FILE),
)

mkd_dump = _so.mkd_dump
mkd_dump.argtypes = (
    ctypes.POINTER(Document),
    ctypes.POINTER(FILE),
    ctypes.c_int,
    ctypes.c_char_p,
)

mkd_cleanup = _so.mkd_cleanup
mkd_cleanup.argtypes = (ctypes.POINTER(Document),)

mkd_doc_title = _so.mkd_doc_title
mkd_doc_title.argtypes = (ctypes.POINTER(Document),)
mkd_doc_title.restype = ctypes.c_char_p

mkd_doc_author = _so.mkd_doc_author
mkd_doc_author.argtypes = (ctypes.POINTER(Document),)
mkd_doc_author.restype = ctypes.c_char_p

mkd_doc_date = _so.mkd_doc_date
mkd_doc_date.argtypes = (ctypes.POINTER(Document),)
mkd_doc_date.restype = ctypes.c_char_p



import os
import posixpath
import shutil
import subprocess
import urllib2

from distutils.command.build_ext import build_ext as _build_ext
from distutils.core import setup, Extension


DISCOUNT_VERSION = '1.5.8'

DISCOUNT_CONFIGURATION = (
    # Use pandoc-style header blocks
    '--enable-pandoc-header',

    # A^B becomes A<sup>B</sup>
    '--enable-superscript',

    # underscores aren'\''t special in the middle of words
    '--relaxed-emphasis',

    # Set tabstops to N characters (default is 4)
    '--with-tabstops=4',

    # Enable >%id% divisions
    '--enable-div',

    # Enable (a)/(b)/(c) lists
    '--enable-alpha-list',

    # Enable memory allocation debugging
    # '--enable-amalloc',

    # Turn on all stable optional features
    # '--enable-all-features',
)

# TODO: host on pypi
DISCOUNT_DOWNLOAD_URL = (
    'http://www.pell.portland.or.us/'
    '~orc/Code/discount/discount-%s.tar.gz'
) % DISCOUNT_VERSION


class build_ext(_build_ext):
    user_options = _build_ext.user_options + [
        ('discount-src-path=', None,
         'User-defined path to discount source files.'),
    ]

    def initialize_options(self):
        _build_ext.initialize_options(self)
        self.discount_src_path = None

    def build_extension(self, ext):
        if ext.name == '_discount':
            self.build_extension_discount(ext)
        else:
            _build_ext.build_extension(self, ext)

    def build_extension_discount(self, ext):
        if self.discount_src_path is None:
            filepath = os.path.join(
                self.build_temp,
                posixpath.basename(DISCOUNT_DOWNLOAD_URL)
            )
            if not os.path.lexists(self.build_temp):
                os.makedirs(self.build_temp)

            if not os.path.exists(filepath):
                print 'Downloading %s...' % DISCOUNT_DOWNLOAD_URL

                data = urllib2.urlopen(DISCOUNT_DOWNLOAD_URL)
                fp = open(filepath, 'wb')
                fp.write(data.read())
                fp.close()

                print 'Extracting %s...' % filepath

                subprocess.call(
                    ['tar', 'xzf', filepath, '-C', self.build_temp]
                )

            discount_src_path = filepath[:-len('.tar.gz')]

        else:
            discount_src_path = self.discount_src_path

        if not os.path.exists(
            os.path.join(discount_src_path, 'config.h')):
            current_dir = os.getcwd()
            os.chdir(discount_src_path)
            subprocess.call(
                ('./configure.sh',) + DISCOUNT_CONFIGURATION,
                env=os.environ
            )
            os.chdir(current_dir)

        ext.sources = [
            os.path.join(discount_src_path, s) for s in ext.sources
        ]
        ext.extra_compile_args += [
            '-I%s' % discount_src_path,
        ]

        _build_ext.build_extension(self, ext)

        ext_filename = self.get_ext_filename(ext.name)

        if not os.path.exists(ext_filename):
            # Copy the shared library to same dir as setup.py for
            # convenience, helpful for running test suite without
            # installing package.
            shutil.copy(
                os.path.join(self.build_lib, ext_filename),
                ext_filename
            )


setup(
    name='discount',
    license='BSD',
    version='0.1.0BETA',

    author='Trapeze Media',
    author_email='technology@trapeze.com',
    url="http://github.com/trapeze/python-discount",
    download_url='http://pypi.python.org/pypi/discount',
    description='A Python interface for Discount, the C Markdown parser',
    long_description=open('README.rst').read(),
    keywords='markdown discount ctypes',

    provides=[
        'discount',
    ],

    py_modules=[
        'discount',
        'discount.libmarkdown',
    ],

    ext_modules=[
        Extension(
            '_discount',
            extra_compile_args=[
                '-DVERSION="%s"' % DISCOUNT_VERSION,
            ],
            sources=[
                'Csio.c',
                'amalloc.c',
                'css.c',
                'docheader.c',
                'dumptree.c',
                'generate.c',
                'markdown.c',
                'mkdio.c',
                'resource.c',
                'theme.c',
                'toc.c',
                'version.c',
                'xml.c',
                'xmlpage.c',
            ],
        )
    ],

    cmdclass={
        'build_ext': build_ext
    },

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: C',
        'Programming Language :: Python',
        'Topic :: Text Processing :: Markup'
    ],
)
